/***************************************************************************
                          longcode.c  -  description
                             -------------------
    begin                : Sat Jul 28 2001
    copyright            : (C) 2001 by cmeclax
    email                : cmeclax@ixazon.dynip.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Implements the --longcode option. This has two effects: codes are
   output in long form, and codes are compared by their histograms,
   not their bit patterns. The long code format is as follows:
   Take the lowest bar in the histogram. Encode it in base 3,
   using 00, 01, 10 as trits, and follow it with 11. Encode the range
   (highest bar minus lowest bar) in the same manner. From the
   range, determine a code for 0..range as follows:
   Range Number Code         Range Number Code
   0     0                   5     0      011
   1     0      0                  1      111
         1      1                  2      00
   2     0      11                 3      10
         1      0                  4      010
         2      10                 5      110
   3     0      00           6     0      101
         1      10                 1      011
         2      01                 2      111
         3      11                 3      00
   4     0      111                4      100
         1      00                 5      010
         2      10                 6      110
         3      01
         4      110
   Encode the difference between each of the 256 bars and the lowest.
   Pack six bits to a character using modified base 64 code, rearranged
   so that no hex character corresponds to a number with two
   consecutive 1 bits, except for 0, which stands for 111111 and
   can be prepended to the code without changing its meaning.

   The method of comparing codes using their histograms is as follows:
   Subtract the mean from each bar. Multiply the two histograms termwise
   and add them up; square each histogram termwise and add the terms.
   The nilsimsa is the dot product of the two histograms divided by
   the square root of the product of the dot product of each histogram
   with itself. Then multiply this number by 128 and round to the
   nearest integer.

   Base 64 code table:
       0 1 2 3   4 5 6 7   8 9 a b   c d e f
   00  0 0 0 A   0 0 a A   0 0 0 A   A A A A
   10  0 0 0 A   0 0 a A   a a a A   A A A A
   20  0 0 0 A   0 0 a A   0 0 0 A   A A A A
   30  A A A A   A A A A   A A A A   A A A X
       0 1 2 3   4 5 6 7   8 9 a b   c d e f
   00  1 2 3 j   a d g k   A B C l   m n o p
   10  4 5 6 q   b e h r   G H I s   t u v w
   20  7 8 9 x   c f i y   D E F z   J K L M
   30  + / N O   P Q R S   T U V W   X Y Z 0
   */

int codebase,codebits,codebufferbits,codebufferlen;
unsigned char *codebuffer;

unsigned char enctable[64]="123jadgkABClmnop456qbehrGHIstuvw789xcfiyDEFzJKLM+/NOPQRSTUVWXYZ0";
unsigned char dectable[256];

void fill_dectable()
{int i;
 for (i=0;i<256;i++)
     dectable[i]=255;
 for (i=0;i<64;i++)
     dectable[enctable[i]]=i;
 }

void pack(unsigned bits,int len)
{unsigned acc;
 int newcodebufferbits,accbits;
 unsigned char *codewrite;
 newcodebufferbits=codebufferbits+bits;
 if ((newcodebufferbits+5)/6>codebufferlen)
    {codebuffer=realloc(codebuffer,codebufferlen+=16);
     if (!codebuffer)
        /* out of memory */;
     memset(codebuffer+codebufferlen-16,0,16);
     }
 codewrite=codebuffer+(codebufferbits/6);
 acc=*codewrite;
 accbits=codebufferbits%6;
 acc+=(bits<<accbits);
 accbits+=len;
 while (accbits>0)
    {*codewrite++=acc&0x3f;
     accbits-=6;
     }
 codebufferbits=newcodebufferbits;
 }

